/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <iostream>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <interface/XPathNSResolverImpl.hpp>
#include <interface/XPathResultImpl.hpp>
#include <interface/XPathEvaluatorImpl.hpp>
#include <DOMutils/XStr.hpp>
#include <dom3-xpath/Pathan.hpp>
#include "DOMSerialise.hpp"

void serialiseResults(const XPathResult *snapshotResult, const XPathResult *singleNodeResult);

using namespace std;

int main()
{
        
  // Initialize the XML4C2 system
    
  try {
    XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
  }
  catch(const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
    cerr << "Error during Xerces-c Initialization.\n"
	 << "  Exception message:"
	 << XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage()) << endl;
    return 1;
  }

  XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser *xmlParser = new XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser();

  xmlParser->setDoNamespaces(true);
  xmlParser->parse("data.xml");

  const XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *testDoc = xmlParser->getDocument();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *tmpNode = testDoc->getDocumentElement();

  try {
    XPathEvaluator *evaluator = new XPathEvaluatorImpl();
  
    //Do a number of tests requesting a single node or a snapshot and
    //check that the first node of the snapshot is the same as the
    //single node
    XPathResult *snapshotResult, *singleNodeResult;

    XPathNSResolver *docNSResolver = evaluator->createNSResolver(tmpNode);

    // /child::*[1]/textTest/text()

    XPathExpression *parsedExpression = evaluator->createExpression(X("/child::*[1]/textTest/text()"), docNSResolver);

    snapshotResult = parsedExpression->evaluate(tmpNode, XPathResult::ORDERED_NODE_SNAPSHOT_TYPE, snapshotResult);
    singleNodeResult = parsedExpression->evaluate(tmpNode, XPathResult::FIRST_ORDERED_NODE_TYPE, singleNodeResult);

    serialiseResults(snapshotResult, singleNodeResult);

    // /child::*[1]/*

    parsedExpression = evaluator->createExpression(X("/child::*[1]/*"), docNSResolver);

    snapshotResult = parsedExpression->evaluate(tmpNode, XPathResult::ORDERED_NODE_SNAPSHOT_TYPE, snapshotResult);
    singleNodeResult = parsedExpression->evaluate(tmpNode, XPathResult::FIRST_ORDERED_NODE_TYPE, singleNodeResult);

    serialiseResults(snapshotResult, singleNodeResult);

    // /da:data[1]/element2[1]/*

    parsedExpression = evaluator->createExpression(X("/da:data[1]/element2[1]"), docNSResolver);

    snapshotResult = parsedExpression->evaluate(tmpNode, XPathResult::ORDERED_NODE_SNAPSHOT_TYPE, snapshotResult);
    singleNodeResult = parsedExpression->evaluate(tmpNode, XPathResult::FIRST_ORDERED_NODE_TYPE, singleNodeResult);

    serialiseResults(snapshotResult, singleNodeResult);

    // /da:data[1]/element2[1]/ancestor::*

    parsedExpression = evaluator->createExpression(X("/da:data[1]/element2[1]/ancestor::*"), docNSResolver);

    snapshotResult = parsedExpression->evaluate(tmpNode, XPathResult::ORDERED_NODE_SNAPSHOT_TYPE, snapshotResult);
    singleNodeResult = parsedExpression->evaluate(tmpNode, XPathResult::FIRST_ORDERED_NODE_TYPE, singleNodeResult);

    serialiseResults(snapshotResult, singleNodeResult);

  }
  catch(const XPathException &e) {
    cout << "XPathException: " << e.getString() << endl;
    return 1;
  }
  catch(const XERCES_CPP_NAMESPACE_QUALIFIER DOMException &e) {
    cout << "XERCES_CPP_NAMESPACE_QUALIFIER DOMException " << e.code << " occured: " << e.msg << endl;
    return 1;
  }
  
  return 0;
}//main

void serialiseResults(const XPathResult *snapshotResult, const XPathResult *singleNodeResult) {

    cout << "ORDERED_NODE_SNAPSHOT_TYPE" << endl;
    cout << snapshotResult->snapshotItem(0) << endl;

    cout << "FIRST_ORDERED_NODE_TYPE" << endl;
    cout << singleNodeResult->getSingleNodeValue() << endl;
}
